      program mcl2stc
c     reads mcluster output
c     generates celestia stc catalog file

      character*512 line
      character*20 fields(21),filename,ara0,adec0,adist0,cluname
      integer ntypes(0:20)
      character*10 atypes(0:20)
      parameter (nhr=1300)

      character*6 atype
      real smr(nhr),slr(nhr),sam(nhr),rad(nhr)
      double precision x,y,z

c        0 - deeply or fully convective low mass MS star
c        1 - Main Sequence star
c        2 - Hertzsprung Gap
c        3 - First Giant Branch
c        4 - Core Helium Burning
c        5 - First Asymptotic Giant Branch
c        6 - Second Asymptotic Giant Branch
c        7 - Main Sequence Naked Helium star
c        8 - Hertzsprung Gap Naked Helium star
c        9 - Giant Branch Naked Helium star
c       10 - Helium White Dwarf
c       11 - Carbon/Oxygen White Dwarf
c       12 - Oxygen/Neon White Dwarf
c       13 - Neutron Star
c       14 - Black Hole
c       15 - Massless Supernova

      data atypes/'low mass','main seq','hertz gap','1st gnt br',
      1 'corehelium','1st as.gnt','2nd as.gnt','m.seq.hel.',
      1 'hgap hel.','gnt.br hel','hel wd','co wd','on wd',
      1 'neut. star','black hole','snr',5*'-'/
      data ntypes/21*0/
      data smr,slr,sam/nhr*0.0,nhr*0.0,nhr*0.0/

c===============================================================
c get control params:
c name of input and output files, cluster name, ra0, dec0, dist0

      call get_args(filename, cluname, ara0,adec0,adist0)
      call str_len(filename,lenf)
      call str_len(cluname,lenc)

c===============================================================
c     process cluster star table
      open(unit=3,file=filename(:lenf)//'.txt',
      1    form='formatted',type='old',recl=512)

           xmax = 0.0
           xmin = 1e6
           n=0
           ndim=0

 100       continue
           read(3,fmt='(a)',end=30) line
           n=n+1

c mcluster:
c mass, positions (x,y,z), velocities (vx,vy,vz) [7]
c sse:
c ZAMS mass, stellar type, epoch, spin, radius, luminosity, ageMy, [7]
c metallicity, [1]
c absVmag, appVmag, B-V, eff.temp(K), random err Vmag, ran.err B-V [6]
c
c last6: obs w/8m-class tel from dist Rgal.

c1.51347017
c       123456789012345678901234567890
c	-0.0478685230486345 -0.9766166901130282 -0.5706268349650133
c	0.3559244315556876 3.0327281565223374 -0.9735894942474048
c	1.51347017	1	0.00000000	1758.38230351	1.66610347
c	5.93262429	0.00000000	0.02000000	2.96939758
c	17.96939758	0.32365601	6987.08706371	0.02386778
c	0.02166111

      call fixtab(line)
      call extr_tab_fields (line,fields)
d      print *, 'star # ',n
d      print *, ( 'f',i,': ',fields(i),i=1,21)

c     discover mass range of stars
      read(fields(1),fmt='(f20.10)') xmass
      if (xmass.lt.xmin) xmin = xmass
      if (xmass.gt.xmax) xmax = xmass

c     count mcluster types of stars
      call str_len(fields(9),nc)
      if (nc.eq.1) then
         read(fields(9),fmt='(i1)') ntype
      elseif (nc.eq.2) then
         read(fields(9),fmt='(i2)') ntype
      else
         print *, 'nc = ',nc
      endif

d      print *,n,nc, ntype
      ntypes(ntype)=ntypes(ntype)+1

      read(fields(12),fmt='(f20.10)') xrad

      read(fields(13),fmt='(f20.10)') xlum
      read(fields(16),fmt='(f20.10)') vmag
      read(fields(19),fmt='(f20.10)') t

d     print *, xmass, xlum,vmag,t
      if (xmag.gt.9000.0) ndim=ndim+1


c     guess type of star
      call guess_type(n,xmass,xlum,vmag,xrad,t,ntype,atype)
d      print *, ' type = ',atype

      read(fields(2),fmt='(d20.15)') x
      read(fields(3),fmt='(d20.15)') y
      read(fields(4),fmt='(d20.15)') z

c      print *, fields(2)
c      write(6,fmt='(1x,d23.17)') x

      if (atype.ne.'dead') then
      call gen_stc(x,y,z,atype,vmag,
      1 ara0,adec0,adist0,filename,cluname)
        endif

c     count spectral types
        call count_stype (atype)
      goto 100

 30   continue
      close(3)
      close(4)

      print *,'Star Types'
      do i=0,15
         print *, atypes(i),ntypes(i)
      enddo
      print *, 'mass min,max = ',xmin,', ',xmax
c      print *, 'too dim: ',ndim

      call show_stypes
      end
c-----------------------------------------------------
      subroutine gen_stc(x,y,z,atype,xmag,
      1 ara0,adec0,adist0,filename,cluname)

c     translates mcluster's stellar types into spectral types
c     and xyz to ra,dec,distance.

      character*20 ara0,adec0,adist0,filename,cluname
      character*6 ans
      character*5 atype
      logical first
      double precision x0,y0,z0,ra0,dec0,dist0,x,y,z,r,dx,dy,dz
      double precision ra,dec,dist,x1,y1,z1,pc2ly

      save x0,y0,z0,ns,first,lenc,pc2ly
      data pc2ly/3.26163626d0/

      data first /.true./
      if (first) then
         first = .false.

         call str_len(filename,lenf)
         call str_len(cluname,lenc)

      open(unit=4,file=filename(:lenf)//'.stc',
      1 form='formatted',recl=512)
         ns = 0

c     position of cluster center

c         ra0 = (5.3928d0) *15.0d0
c         dec0 = -69.7561d0
c         dist0 = 1.631d+05

         read(ara0,fmt='(d20.10)') ra0
         read(adec0,fmt='(d20.10)') dec0
         read(adist0,fmt='(d20.10)') dist0
         call sph2cart (ra0,dec0,dist0, x0,y0,z0)


c         print *, x0,y0,z0
c      r = sqrt(x0**2+y0**2+z0**2)
c         print *, 'rc= ',r

      endif

      call str_len(atype,na)
      if (atype(:na).eq.'dead')   return

      ns = ns+1
      write (ans,fmt='(i6.6)') ns

d      print *, x,y,z
d      r = sqrt(x**2+y**2+z**2)
d      print *, 'rs= ',r

      dx = x*pc2ly+x0
      dy = y*pc2ly+y0
      dz = z*pc2ly+z0

      if (.false.) then
c     verify no loss in precision
c     identical results have confirmed accuracy
      PRINT *, ' '
      write(6,fmt='(1x,d20.15)') x
      write(6,fmt='(1x,d20.15)') y
      write(6,fmt='(1x,d20.15)') z
      call cart2sph (x,y,z,ra,dec,dist)
      call sph2cart2 (ra,dec,dist, x1,y1,z1)
      PRINT *, ' '
      write(6,fmt='(1x,d20.15)') x1
      write(6,fmt='(1x,d20.15)') y1
      write(6,fmt='(1x,d20.15)') z1
         stop
      endif

      call cart2sph (dx,dy,dz,ra,dec,dist)

c 33   format(1x,6(a,d23.17))
c 33   format(1x,6(a,d22.15))
 33   format(1x,a,d23.17,a,d23.17,a,d23.17,a,f7.2,a)

      if (atype(:na).eq.'dead') then
d         return
      write(4,fmt=33) '# Star "'//cluname(:lenc)//ans//'" {'//
      1 '  RA ',ra,' Dec ',dec,' Distance ',dist,' SpectralType "'
      1//atype(:na)//'" AbsMag ',xmag,' }'

         return
         else

      write(4,fmt=33) 'Star "'//cluname(:lenc)//ans//'" {'//
      1 'RA ',ra,' Dec ',dec,' Distance ',dist,' SpectralType "'
      1//atype(:na)//'" AbsMag ',xmag,' }'

         endif

c      if (ns.gt.5) stop
      return
      end

c==============================================
	subroutine sph2cart (ra,dec,r, x,y,z)
c     phi = longitude
c     theta = latitude

        implicit double precision (a-z)

	d2r = 6.283185307179586476925287D0/360.0d0
	theta = d2r* (dec)
	phi = d2r* (ra)

	x = r*cos(theta)*cos(phi)
        y = r * cos(theta)*sin(phi)
        z = r * sin(theta)

	return

	end

c==============================================
	subroutine sph2cart2 (ra,dec,r, x,y,z)
c     phi = longitude
c     theta = latitude

        implicit double precision (a-z)

	d2r = 6.283185307179586476925287D0/360.0d0
	theta = d2r* (dec)
	phi = d2r* (ra)

	x = r * sin(theta)*cos(phi)
        y = r * sin(theta)*sin(phi)
        z = r * cos(theta)

	return

	end
c==============================================
	subroutine cart2sph (x,y,z, ra,dec,r)
c     phi = longitude
c     theta = latitude

        implicit double precision (a-z)

d        print *, x,y,z
	d2r = 6.283185307179586476925287D0/360.0d0

        dr = sqrt(x**2+y**2+z**2)
        dtheta = acos(z/dr)
        dphi = atan2(y,x)

        r = dr
        ra = dphi/d2r
        dec = dtheta/d2r
        if (dec.gt.90.0d0) dec = 90.0d0-dec

	return

	end

c-----------------------------------------------------
      subroutine show_stypes
      character*5 atype,stypes(2000)
      integer ntypes(2000)
      common/ctyp/stypes,ntypes,nstype
      ntot = 0

      call aisort(stypes,ntypes,nstype,2)
      print *, 'Sorted by Spectral Type'
      do n=2,nstype
         print *,stypes(n),'  ',ntypes(n)
         ntot = ntot+ntypes(n)
      enddo

      call iasort(ntypes,stypes,nstype,2)

      print *, 'Sorted by Frequency'
      do n=2,nstype
         print *,stypes(n),'  ',ntypes(n)
      enddo

      print *, '# spec. types=',nstype,', total stars=',ntot,
      1' (',ntot-ntypes(nstype),' passed)'
      return
      end

c-----------------------------------------------------
      subroutine count_stype(atype)
      character*5 atype,stypes(2000)
      integer ntypes(2000)
      common/ctyp/stypes,ntypes,nstype
      data nstype /1/

      do nt = 1,nstype
         if (stypes(nt).eq.atype) then
            ntypes(nt)=ntypes(nt)+1
            return
         endif
      enddo
        
      nstype=nstype+1
      stypes(nstype)=atype
      ntypes(nstype)=1
      return
      end

c-----------------------------------------------------
      subroutine guess_type(n,xmass,xlum,xmag,xrad,t,ntype,
      1 atype)

c     STELLAR TYPES - ntype
c
c        0 - deeply or fully convective low mass MS star
c        1 - Main Sequence star
c        2 - Hertzsprung Gap
c        3 - First Giant Branch
c        4 - Core Helium Burning
c        5 - First Asymptotic Giant Branch
c        6 - Second Asymptotic Giant Branch
c        7 - Main Sequence Naked Helium star
c        8 - Hertzsprung Gap Naked Helium star
c        9 - Giant Branch Naked Helium star
c       10 - Helium White Dwarf
c       11 - Carbon/Oxygen White Dwarf
c       12 - Oxygen/Neon White Dwarf
c       13 - Neutron Star
c       14 - Black Hole
c       15 - Massless Supernova

      character*1 class(11),ac
      character*2 ac2
      character*4 at
      character*5 aclass,clum,clums(6)
      character*7 atype

      real tclass(11),rlums(6)


c     luminosity classes
c 

c     source for ranges: plot at
c     http://stars.astro.illinois.edu/sow/hrd.html
c and

      data clums/ 'I-a0','I-a','I-b', 'II', 'III', 'IV'/
c      data rlums/ -8.5, -6.5, -4.5, -2.0,  1.5,  3.5/
c     solar units
      data rlums/ 200000.0, 20000.0, 3000.0, 400.0,  11.5,  4.0/



c     class temperatures
c     source: 
c     http://hyperphysics.phy-astr.gsu.edu/hbase/starlog/staspe.html
c and 
c     http://en.wikipedia.org/wiki/Stellar_classification

      data class /      'x',    'O',   'B',   'A',   'F',   'G',
      1                'K',    'M',   'L',   'T', 'Y'/

      data tclass /60000.0, 30000.0,10000.0, 7500.0, 6000.0, 5000.0,
      1             3500.0,  2000.0, 1300.0,  700.0, 0.0/

c    abs mag is 20% too dim for G2 stars?? (evidence: lack of Sun-like stars)
c      hack: decrease abs mag --> increase brightness
       xmag = 0.95*xmag


c     don't waste time on dead stars
      if (xmag.gt.9000.0) then
         atype = 'dead'
         return
      endif

      atype = '????' ! if star type >15

c     Spectral class
      do nc = 2,10
         if (t .gt.tclass(nc)) then
            ac = class(nc)

c     0-9 approximately linear within class
c     source: plot at 
c     http://www.rwc.uc.edu/koehler/astrophys/populations.html

            csiz = tclass(nc-1)-tclass(nc)
            cdel = t - tclass(nc)
            cfrac = cdel/csiz
            dt = 10.0-( 10.0* cfrac)
            nt = dt
d            print *, csiz,cdel,cfrac,dt,nt
            write(at,fmt='(i1)')nt
            aclass = ac//at
            goto 1
         endif
      enddo
 1    continue

d      print *, 'star # ', n, t,' = ',ac,at


c     process each star type
c     determine luminosity class

      clum = ' '

c     main sequence
      if (ntype.le.1 .or. ntype.eq.7) then
         clum = 'V'
         call str_len(aclass,na)
         atype = aclass(:na)//clum
         RETURN

c     giants
      elseif (ntype.le.9.) then
c     giants unlikely to be V, but...
         clum = 'V'
         do nl = 1,6
            if (xlum.gt.rlums(nl)) then
               clum = clums(nl)
               call str_len(aclass,na)
               atype = aclass(:na)//clum
d               print *, n,': ',ntype,xlum,' ',clums(nl),' = ',atype
               RETURN
            endif
         enddo
         call str_len(aclass,na)
         atype = aclass(:na)//clum
d               print *, n,': ',ntype,xlum,' = ',atype
         RETURN

c     white dwarf stars
      elseif (ntype.eq.10) then
c     helium
         dt = 50400.0/T
         write (at,fmt='(f4.1)') dt
         if (at(1:1).eq. ' ') at=at(2:)
         atype = 'DZ'//at(1:1)
         RETURN

      elseif (ntype.eq.11 .or. ntype.eq.12) then
c     metals: oxygen etc
         dt = 50400.0/T
         write (at,fmt='(f4.1)') dt
         if (at(1:1).eq. ' ') at=at(2:)
         atype = 'DZ'//at(1:1)
         RETURN

c     black hole --> neutron star
        elseif (ntype.eq.13 .or. ntype.eq.14) then
           atype = 'Q'
           RETURN

        elseif (ntype.eq.15) then
           atype = 'SNR'
           RETURN
        endif

      return
      end

c-----------------------------------------------------
      subroutine fixtab(line)
      character*512 line
      do nc=1,512
         if (line(nc:nc).eq.' ') line(nc:nc)=char(9)
         enddo
      return
      end

c-----------------------------------------------------
      subroutine extr_tab_fields (line,fields)
c     separated by tabs
      character*20 fields(21)
      character*512 line,temp
      temp = line
      do ntab = 1,20
         ipc = index(temp,char(9))
         if (ipc.gt.0) then
            fields(ntab)=temp(:ipc-1)
            temp = temp(ipc+1:)
         else
            fields(ntab) = ' '
         endif
         enddo
      fields(21)=temp
      return
      end

c-----------------------------------------------------
      subroutine extr_comma_fields (line,fields)
c     separated by commas
      character*20 fields(21)
      character*512 line,temp
      temp = line
      do ntab = 1,20
         ipc = index(temp,',')
         if (ipc.gt.0) then
            fields(ntab)=temp(:ipc-1)
            temp = temp(ipc+1:)
         else
            fields(ntab) = ' '
         endif
         enddo
      fields(21)=temp
      return
      end

c-----------------------------------------------------
      subroutine str_len(str,ncf)
      character*(*) str
      lens = len(str)
      do nc = lens,1,-1
         if (str(nc:nc).ne. ' ') then
            ncf = nc
            return
         endif
      enddo
      ncf = 1
      return
      end
c==========================================================
      SUBROUTINE IASORT (X, Y, N, KFLAG)
C***BEGIN PROLOGUE  IASORT
C***PURPOSE  Sort an array and optionally make the same interchanges in
C            an auxiliary array.  The array may be sorted in increasing
C            or decreasing order.  A slightly modified QUICKSORT
C            algorithm is used.
C***LIBRARY   SLATEC
C***CATEGORY  N6A2B
C***TYPE      SINGLE PRECISION (SSORT-S, DSORT-D, ISORT-I)
C***KEYWORDS  SINGLETON QUICKSORT, SORT, SORTING
C***AUTHOR  Jones, R. E., (SNLA)
C           Wisniewski, J. A., (SNLA)
C***DESCRIPTION
C
C   SSORT sorts array X and optionally makes the same interchanges in
C   array Y.  The array X may be sorted in increasing order or
C   decreasing order.  A slightly modified quicksort algorithm is used.
C
C   Description of Parameters
C      X - array of values to be sorted   (usually abscissas)
C      Y - array to be (optionally) carried along
C      N - number of values in array X to be sorted
C      KFLAG - control parameter
C            =  2  means sort X in increasing order and carry Y along.
C            =  1  means sort X in increasing order (ignoring Y)
C            = -1  means sort X in decreasing order (ignoring Y)
C            = -2  means sort X in decreasing order and carry Y along.
C
C***REFERENCES  R. C. Singleton, Algorithm 347, An efficient algorithm
C                 for sorting with minimal storage, Communications of
C                 the ACM, 12, 3 (1969), pp. 185-187.
C***REVISION HISTORY  (YYMMDD)
C   761101  DATE WRITTEN
C   761118  Modified to use the Singleton quicksort algorithm.  (JAW)
C   890531  Changed all specific intrinsics to generic.  (WRB)
C   890831  Modified array declarations.  (WRB)
C   891009  Removed unreferenced statement labels.  (WRB)
C   891024  Changed category.  (WRB)
C   891024  REVISION DATE from Version 3.2
C   891214  Prologue converted to Version 4.0 format.  (BAB)
C   900315  CALLs to XERROR changed to CALLs to XERMSG.  (THJ)
C   901012  Declared all variables; changed X,Y to SX,SY. (M. McClain)
C   920501  Reformatted the REFERENCES section.  (DWL, WRB)
C   920519  Clarified error messages.  (DWL)
C   920801  Declarations section rebuilt and code restructured to use
C           IF-THEN-ELSE-ENDIF.  (RWC, WRB)
c   110530   change to character+integer sort (seb)

C***END PROLOGUE  SSORT
C     .. Scalar Arguments ..
      INTEGER KFLAG, N
C     .. Array Arguments ..
      INTEGER X(*),T, TT
      CHARACTER*5 Y(*), TTY, TY
C     .. Local Scalars ..
      REAL R
      INTEGER I, IJ, J, K, KK, L, M, NN
C     .. Local Arrays ..
      INTEGER IL(21), IU(21)
C     .. External Subroutines ..
C     None
C     .. Intrinsic Functions ..
      INTRINSIC ABS, INT
C***FIRST EXECUTABLE STATEMENT  SSORT
      NN = N
      IF (NN .LT. 1) THEN
         PRINT *,
     +      'The number of values to be sorted is not positive.'
         RETURN
      ENDIF
C
      KK = ABS(KFLAG)
      IF (KK.NE.1 .AND. KK.NE.2) THEN
         PRINT *,
     +      'The sort control parameter, K, is not 2, 1, -1, or -2.'
         RETURN
      ENDIF
C
C     Alter array X to get decreasing order if needed
C
      IF (KFLAG .LE. -1) THEN
         PRINT *, '? NEG KFLAG doesnt work for chars'
c         DO 10 I=1,NN
c            X(I) = -X(I)
c   10    CONTINUE
      ENDIF
C
      IF (KK .EQ. 2) GO TO 100
C
C     Sort X only
C
      M = 1
      I = 1
      J = NN
      R = 0.375E0
C
   20 IF (I .EQ. J) GO TO 60
      IF (R .LE. 0.5898437E0) THEN
         R = R+3.90625E-2
      ELSE
         R = R-0.21875E0
      ENDIF
C
   30 K = I
C
C     Select a central element of the array and save it in location T
C
      IJ = I + INT((J-I)*R)
      T = X(IJ)
C
C     If first element of array is greater than T, interchange with T
C
      IF (X(I) .GT. T) THEN
         X(IJ) = X(I)
         X(I) = T
         T = X(IJ)
      ENDIF
      L = J
C
C     If last element of array is less than than T, interchange with T
C
      IF (X(J) .LT. T) THEN
         X(IJ) = X(J)
         X(J) = T
         T = X(IJ)
C
C        If first element of array is greater than T, interchange with T
C
         IF (X(I) .GT. T) THEN
            X(IJ) = X(I)
            X(I) = T
            T = X(IJ)
         ENDIF
      ENDIF
C
C     Find an element in the second half of the array which is smaller
C     than T
C
   40 L = L-1
      IF (X(L) .GT. T) GO TO 40
C
C     Find an element in the first half of the array which is greater
C     than T
C
   50 K = K+1
      IF (X(K) .LT. T) GO TO 50
C
C     Interchange these elements
C
      IF (K .LE. L) THEN
         TT = X(L)
         X(L) = X(K)
         X(K) = TT
         GO TO 40
      ENDIF
C
C     Save upper and lower subscripts of the array yet to be sorted
C
      IF (L-I .GT. J-K) THEN
         IL(M) = I
         IU(M) = L
         I = K
         M = M+1
      ELSE
         IL(M) = K
         IU(M) = J
         J = L
         M = M+1
      ENDIF
      GO TO 70
C
C     Begin again on another portion of the unsorted array
C
   60 M = M-1
      IF (M .EQ. 0) GO TO 190
      I = IL(M)
      J = IU(M)
C
   70 IF (J-I .GE. 1) GO TO 30
      IF (I .EQ. 1) GO TO 20
      I = I-1
C
   80 I = I+1
      IF (I .EQ. J) GO TO 60
      T = X(I+1)
      IF (X(I) .LE. T) GO TO 80
      K = I
C
   90 X(K+1) = X(K)
      K = K-1
      IF (T .LT. X(K)) GO TO 90
      X(K+1) = T
      GO TO 80
C
C     Sort X and carry Y along
C
  100 M = 1
      I = 1
      J = NN
      R = 0.375E0
C
  110 IF (I .EQ. J) GO TO 150
      IF (R .LE. 0.5898437E0) THEN
         R = R+3.90625E-2
      ELSE
         R = R-0.21875E0
      ENDIF
C
  120 K = I
C
C     Select a central element of the array and save it in location T
C
      IJ = I + INT((J-I)*R)
      T = X(IJ)
      TY = Y(IJ)
C
C     If first element of array is greater than T, interchange with T
C
      IF (X(I) .GT. T) THEN
         X(IJ) = X(I)
         X(I) = T
         T = X(IJ)
         Y(IJ) = Y(I)
         Y(I) = TY
         TY = Y(IJ)
      ENDIF
      L = J
C
C     If last element of array is less than T, interchange with T
C
      IF (X(J) .LT. T) THEN
         X(IJ) = X(J)
         X(J) = T
         T = X(IJ)
         Y(IJ) = Y(J)
         Y(J) = TY
         TY = Y(IJ)
C
C        If first element of array is greater than T, interchange with T
C
         IF (X(I) .GT. T) THEN
            X(IJ) = X(I)
            X(I) = T
            T = X(IJ)
            Y(IJ) = Y(I)
            Y(I) = TY
            TY = Y(IJ)
         ENDIF
      ENDIF
C
C     Find an element in the second half of the array which is smaller
C     than T
C
  130 L = L-1
      IF (X(L) .GT. T) GO TO 130
C
C     Find an element in the first half of the array which is greater
C     than T
C
  140 K = K+1
      IF (X(K) .LT. T) GO TO 140
C
C     Interchange these elements
C
      IF (K .LE. L) THEN
         TT = X(L)
         X(L) = X(K)
         X(K) = TT
         TTY = Y(L)
         Y(L) = Y(K)
         Y(K) = TTY
         GO TO 130
      ENDIF
C
C     Save upper and lower subscripts of the array yet to be sorted
C
      IF (L-I .GT. J-K) THEN
         IL(M) = I
         IU(M) = L
         I = K
         M = M+1
      ELSE
         IL(M) = K
         IU(M) = J
         J = L
         M = M+1
      ENDIF
      GO TO 160
C
C     Begin again on another portion of the unsorted array
C
  150 M = M-1
      IF (M .EQ. 0) GO TO 190
      I = IL(M)
      J = IU(M)
C
  160 IF (J-I .GE. 1) GO TO 120
      IF (I .EQ. 1) GO TO 110
      I = I-1
C
  170 I = I+1
      IF (I .EQ. J) GO TO 150
      T = X(I+1)
      TY = Y(I+1)
      IF (X(I) .LE. T) GO TO 170
      K = I
C
  180 X(K+1) = X(K)
      Y(K+1) = Y(K)
      K = K-1
      IF (T .LT. X(K)) GO TO 180
      X(K+1) = T
      Y(K+1) = TY
      GO TO 170
C
C     Clean up
C
  190 IF (KFLAG .LE. -1) THEN
c         DO 200 I=1,NN
c            X(I) = -X(I)
c  200    CONTINUE
      ENDIF
      RETURN
      END
c==========================================================
      SUBROUTINE AISORT (X, Y, N, KFLAG)
C***BEGIN PROLOGUE  AISORT
C***PURPOSE  Sort an array and optionally make the same interchanges in
C            an auxiliary array.  The array may be sorted in increasing
C            or decreasing order.  A slightly modified QUICKSORT
C            algorithm is used.
C***LIBRARY   SLATEC
C***CATEGORY  N6A2B
C***TYPE      SINGLE PRECISION (SSORT-S, DSORT-D, ISORT-I)
C***KEYWORDS  SINGLETON QUICKSORT, SORT, SORTING
C***AUTHOR  Jones, R. E., (SNLA)
C           Wisniewski, J. A., (SNLA)
C***DESCRIPTION
C
C   SSORT sorts array X and optionally makes the same interchanges in
C   array Y.  The array X may be sorted in increasing order or
C   decreasing order.  A slightly modified quicksort algorithm is used.
C
C   Description of Parameters
C      X - array of values to be sorted   (usually abscissas)
C      Y - array to be (optionally) carried along
C      N - number of values in array X to be sorted
C      KFLAG - control parameter
C            =  2  means sort X in increasing order and carry Y along.
C            =  1  means sort X in increasing order (ignoring Y)
C            = -1  means sort X in decreasing order (ignoring Y)
C            = -2  means sort X in decreasing order and carry Y along.
C
C***REFERENCES  R. C. Singleton, Algorithm 347, An efficient algorithm
C                 for sorting with minimal storage, Communications of
C                 the ACM, 12, 3 (1969), pp. 185-187.
C***REVISION HISTORY  (YYMMDD)
C   761101  DATE WRITTEN
C   761118  Modified to use the Singleton quicksort algorithm.  (JAW)
C   890531  Changed all specific intrinsics to generic.  (WRB)
C   890831  Modified array declarations.  (WRB)
C   891009  Removed unreferenced statement labels.  (WRB)
C   891024  Changed category.  (WRB)
C   891024  REVISION DATE from Version 3.2
C   891214  Prologue converted to Version 4.0 format.  (BAB)
C   900315  CALLs to XERROR changed to CALLs to XERMSG.  (THJ)
C   901012  Declared all variables; changed X,Y to SX,SY. (M. McClain)
C   920501  Reformatted the REFERENCES section.  (DWL, WRB)
C   920519  Clarified error messages.  (DWL)
C   920801  Declarations section rebuilt and code restructured to use
C           IF-THEN-ELSE-ENDIF.  (RWC, WRB)
c   110530   change to character+integer sort (seb)

C***END PROLOGUE  SSORT
C     .. Scalar Arguments ..
      INTEGER KFLAG, N
C     .. Array Arguments ..
      CHARACTER*5 X(*)
      INTEGER Y(*)
C     .. Local Scalars ..
      CHARACTER*5 T, TT
      REAL R
      INTEGER TTY, TY
      INTEGER I, IJ, J, K, KK, L, M, NN
C     .. Local Arrays ..
      INTEGER IL(21), IU(21)
C     .. External Subroutines ..
C     None
C     .. Intrinsic Functions ..
      INTRINSIC ABS, INT
C***FIRST EXECUTABLE STATEMENT  SSORT
      NN = N
      IF (NN .LT. 1) THEN
         PRINT *,
     +      'The number of values to be sorted is not positive.'
         RETURN
      ENDIF
C
      KK = ABS(KFLAG)
      IF (KK.NE.1 .AND. KK.NE.2) THEN
         PRINT *,
     +      'The sort control parameter, K, is not 2, 1, -1, or -2.'
         RETURN
      ENDIF
C
C     Alter array X to get decreasing order if needed
C
      IF (KFLAG .LE. -1) THEN
         PRINT *, '? NEG KFLAG doesnt work for chars'
c         DO 10 I=1,NN
c            X(I) = -X(I)
c   10    CONTINUE
      ENDIF
C
      IF (KK .EQ. 2) GO TO 100
C
C     Sort X only
C
      M = 1
      I = 1
      J = NN
      R = 0.375E0
C
   20 IF (I .EQ. J) GO TO 60
      IF (R .LE. 0.5898437E0) THEN
         R = R+3.90625E-2
      ELSE
         R = R-0.21875E0
      ENDIF
C
   30 K = I
C
C     Select a central element of the array and save it in location T
C
      IJ = I + INT((J-I)*R)
      T = X(IJ)
C
C     If first element of array is greater than T, interchange with T
C
      IF (X(I) .GT. T) THEN
         X(IJ) = X(I)
         X(I) = T
         T = X(IJ)
      ENDIF
      L = J
C
C     If last element of array is less than than T, interchange with T
C
      IF (X(J) .LT. T) THEN
         X(IJ) = X(J)
         X(J) = T
         T = X(IJ)
C
C        If first element of array is greater than T, interchange with T
C
         IF (X(I) .GT. T) THEN
            X(IJ) = X(I)
            X(I) = T
            T = X(IJ)
         ENDIF
      ENDIF
C
C     Find an element in the second half of the array which is smaller
C     than T
C
   40 L = L-1
      IF (X(L) .GT. T) GO TO 40
C
C     Find an element in the first half of the array which is greater
C     than T
C
   50 K = K+1
      IF (X(K) .LT. T) GO TO 50
C
C     Interchange these elements
C
      IF (K .LE. L) THEN
         TT = X(L)
         X(L) = X(K)
         X(K) = TT
         GO TO 40
      ENDIF
C
C     Save upper and lower subscripts of the array yet to be sorted
C
      IF (L-I .GT. J-K) THEN
         IL(M) = I
         IU(M) = L
         I = K
         M = M+1
      ELSE
         IL(M) = K
         IU(M) = J
         J = L
         M = M+1
      ENDIF
      GO TO 70
C
C     Begin again on another portion of the unsorted array
C
   60 M = M-1
      IF (M .EQ. 0) GO TO 190
      I = IL(M)
      J = IU(M)
C
   70 IF (J-I .GE. 1) GO TO 30
      IF (I .EQ. 1) GO TO 20
      I = I-1
C
   80 I = I+1
      IF (I .EQ. J) GO TO 60
      T = X(I+1)
      IF (X(I) .LE. T) GO TO 80
      K = I
C
   90 X(K+1) = X(K)
      K = K-1
      IF (T .LT. X(K)) GO TO 90
      X(K+1) = T
      GO TO 80
C
C     Sort X and carry Y along
C
  100 M = 1
      I = 1
      J = NN
      R = 0.375E0
C
  110 IF (I .EQ. J) GO TO 150
      IF (R .LE. 0.5898437E0) THEN
         R = R+3.90625E-2
      ELSE
         R = R-0.21875E0
      ENDIF
C
  120 K = I
C
C     Select a central element of the array and save it in location T
C
      IJ = I + INT((J-I)*R)
      T = X(IJ)
      TY = Y(IJ)
C
C     If first element of array is greater than T, interchange with T
C
      IF (X(I) .GT. T) THEN
         X(IJ) = X(I)
         X(I) = T
         T = X(IJ)
         Y(IJ) = Y(I)
         Y(I) = TY
         TY = Y(IJ)
      ENDIF
      L = J
C
C     If last element of array is less than T, interchange with T
C
      IF (X(J) .LT. T) THEN
         X(IJ) = X(J)
         X(J) = T
         T = X(IJ)
         Y(IJ) = Y(J)
         Y(J) = TY
         TY = Y(IJ)
C
C        If first element of array is greater than T, interchange with T
C
         IF (X(I) .GT. T) THEN
            X(IJ) = X(I)
            X(I) = T
            T = X(IJ)
            Y(IJ) = Y(I)
            Y(I) = TY
            TY = Y(IJ)
         ENDIF
      ENDIF
C
C     Find an element in the second half of the array which is smaller
C     than T
C
  130 L = L-1
      IF (X(L) .GT. T) GO TO 130
C
C     Find an element in the first half of the array which is greater
C     than T
C
  140 K = K+1
      IF (X(K) .LT. T) GO TO 140
C
C     Interchange these elements
C
      IF (K .LE. L) THEN
         TT = X(L)
         X(L) = X(K)
         X(K) = TT
         TTY = Y(L)
         Y(L) = Y(K)
         Y(K) = TTY
         GO TO 130
      ENDIF
C
C     Save upper and lower subscripts of the array yet to be sorted
C
      IF (L-I .GT. J-K) THEN
         IL(M) = I
         IU(M) = L
         I = K
         M = M+1
      ELSE
         IL(M) = K
         IU(M) = J
         J = L
         M = M+1
      ENDIF
      GO TO 160
C
C     Begin again on another portion of the unsorted array
C
  150 M = M-1
      IF (M .EQ. 0) GO TO 190
      I = IL(M)
      J = IU(M)
C
  160 IF (J-I .GE. 1) GO TO 120
      IF (I .EQ. 1) GO TO 110
      I = I-1
C
  170 I = I+1
      IF (I .EQ. J) GO TO 150
      T = X(I+1)
      TY = Y(I+1)
      IF (X(I) .LE. T) GO TO 170
      K = I
C
  180 X(K+1) = X(K)
      Y(K+1) = Y(K)
      K = K-1
      IF (T .LT. X(K)) GO TO 180
      X(K+1) = T
      Y(K+1) = TY
      GO TO 170
C
C     Clean up
C
  190 IF (KFLAG .LE. -1) THEN
c         DO 200 I=1,NN
c            X(I) = -X(I)
c  200    CONTINUE
      ENDIF
      RETURN
      END
c=====================================================
      subroutine get_args(filename, cluname, ara0,adec0,adist0)

c get control params:
c name of input and output files, cluster name, ra0, dec0, dist0

      character*20 filename,ara0,adec0,adist0,cluname
      
      call getarg(1,filename)
      if (filename.eq.' ') goto 31

      call getarg(2,cluname)
      if (cluname.eq.' ') goto 31

      call getarg(3,ara0)
      if (ara0.eq.' ') goto 31

      call getarg(4,adec0)
      if (adec0.eq.' ') goto 31

      call getarg(5,adist0)
      if (adist0.eq.' ') goto 31

      call fix_float(ara0)
      call fix_float(adec0)
      call fix_float(adist0)


d      print *, 'args=', filename, cluname,ara0,adec0,adist0
      return

 31   print *, '?MCL2STC -- not enough arguments.'
      print *,' 5 required: filename prefix RA Dec Distance'
      stop
      END

      subroutine fix_float(a0)
c     ensure a decimal point in number
      character*20 a0,tmp

      npd = index(a0,'.')
      if (npd.ne.0) return

      npe = index(a0,'e')
      if (npe.eq.0) then
         call str_len(a0,np)
         a0(np+1:np+2)='.0'
         return
      else
         a0=a0(:npe-1)//'.0'//a0(npe:)
      endif
      return
      end



